@@ -31,6 +31,8 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include <gio/gunixinputstream.h>
+
 #include "camel-mempool.h"
 #include "camel-mime-filter.h"
 #include "camel-mime-parser.h"
@@ -71,8 +73,8 @@ struct _header_scan_state {
 	gchar *outptr;
 	gchar *outend;
 
-	gint fd;			/* input for a fd input */
-	CamelStream *stream;	/* or for a stream */
+	CamelStream *stream;
+	GInputStream *input_stream;
 
 	gint ioerrno;		/* io error state */
 
@@ -138,9 +140,9 @@ struct _header_scan_filter {
 	CamelMimeFilter *filter;
 };
 
+static void folder_scan_reset (struct _header_scan_state *s);
 static void folder_scan_step (struct _header_scan_state *s, gchar **databuffer, gsize *datalength);
 static void folder_scan_drop_step (struct _header_scan_state *s);
-static gint folder_scan_init_with_fd (struct _header_scan_state *s, gint fd);
 static gint folder_scan_init_with_stream (struct _header_scan_state *s, CamelStream *stream, GError **error);
 static struct _header_scan_state *folder_scan_init (void);
 static void folder_scan_close (struct _header_scan_state *s);
@@ -439,12 +441,16 @@ camel_mime_parser_from_line (CamelMimeParser *m)
  * Returns: Returns -1 on error.
  **/
 gint
-camel_mime_parser_init_with_fd (CamelMimeParser *m,
+camel_mime_parser_init_with_fd (CamelMimeParser *parser,
                                 gint fd)
 {
-	struct _header_scan_state *s = _PRIVATE (m);
+	GInputStream *input_stream;
 
-	return folder_scan_init_with_fd (s, fd);
+	input_stream = g_unix_input_stream_new (fd, TRUE);
+	camel_mime_parser_init_with_input_stream (parser, input_stream);
+	g_object_unref (input_stream);
+
+	return 0;
 }
 
 /**
@@ -470,6 +476,51 @@ camel_mime_parser_init_with_stream (CamelMimeParser *parser,
 	return folder_scan_init_with_stream (s, stream, error);
 }
 
+/**
+ * camel_mime_parser_init_with_input_stream:
+ * @parser: a #CamelMimeParser
+ * @input_stream: a #GInputStream
+ *
+ * Initialize the scanner with @input_stream.  The scanner's offsets will
+ * be relative to the current file position of the stream.  As a result,
+ * seekable streams should only be seeked using the parser seek function.
+ *
+ * Since: 3.12
+ **/
+void
+camel_mime_parser_init_with_input_stream (CamelMimeParser *parser,
+                                          GInputStream *input_stream)
+{
+	struct _header_scan_state *s = _PRIVATE (parser);
+
+	folder_scan_reset (s);
+	s->input_stream = g_object_ref (input_stream);
+}
+
+/**
+ * camel_mime_parser_init_with_bytes:
+ * @parser: a #CamelMimeParser
+ * @bytes: a #GBytes containing the message content
+ *
+ * Convenience function creates a #GMemoryInputStream from @bytes and hands
+ * it off to camel_mime_parser_init_with_input_stream().
+ *
+ * Since: 3.12
+ **/
+void
+camel_mime_parser_init_with_bytes (CamelMimeParser *parser,
+                                   GBytes *bytes)
+{
+	GInputStream *input_stream;
+
+	g_return_if_fail (CAMEL_IS_MIME_PARSER (parser));
+	g_return_if_fail (bytes != NULL);
+
+	input_stream = g_memory_input_stream_new_from_bytes (bytes);
+	camel_mime_parser_init_with_input_stream (parser, input_stream);
+	g_object_unref (input_stream);
+}
+
 /**
  * camel_mime_parser_scan_from:
  * @parser: MIME parser object
@@ -874,28 +925,6 @@ camel_mime_parser_stream (CamelMimeParser *parser)
 	return s->stream;
 }
 
-/**
- * camel_mime_parser_fd:
- * @parser: MIME parser object
- *
- * Return the file descriptor, if any, the parser has been
- * initialised with.
- *
- * Should not be read from unless the parser it to terminate,
- * or the seek offset can be reset before the next parse
- * step.
- *
- * Returns: The file descriptor or -1 if the parser
- * is reading from a stream or has not been initialised.
- **/
-gint
-camel_mime_parser_fd (CamelMimeParser *parser)
-{
-	struct _header_scan_state *s = _PRIVATE (parser);
-
-	return s->fd;
-}
-
 /* Return errno of the parser, incase any error occurred during processing */
 gint
 camel_mime_parser_errno (CamelMimeParser *parser)
@@ -931,7 +960,9 @@ folder_read (struct _header_scan_state *s)
 		len = camel_stream_read (
 			s->stream, s->inbuf + inoffset, SCAN_BUF - inoffset, NULL, NULL);
 	} else {
-		len = read (s->fd, s->inbuf + inoffset, SCAN_BUF - inoffset);
+		len = g_input_stream_read (
+			s->input_stream, s->inbuf + inoffset,
+			SCAN_BUF - inoffset, NULL, NULL);
 	}
 	r (printf ("read %d bytes, offset = %d\n", len, inoffset));
 	if (len >= 0) {
@@ -987,7 +1018,17 @@ folder_seek (struct _header_scan_state *s,
 			errno = EINVAL;
 		}
 	} else {
-		newoffset = lseek (s->fd, offset, whence);
+		if (G_IS_SEEKABLE (s->input_stream)) {
+			/* NOTE: assumes whence seekable stream == whence libc, which is probably
+			 * the case (or bloody well should've been) */
+			g_seekable_seek (
+				G_SEEKABLE (s->input_stream),
+				offset, whence, NULL, NULL);
+			newoffset = g_seekable_tell (G_SEEKABLE (s->input_stream));
+		} else {
+			newoffset = -1;
+			errno = EINVAL;
+		}
 	}
 #ifdef PURIFY
 	purify_watch_remove (inend_id);
@@ -1451,11 +1492,8 @@ folder_scan_close (struct _header_scan_state *s)
 	g_free (s->outbuf);
 	while (s->parts)
 		folder_pull_part (s);
-	if (s->fd != -1)
-		close (s->fd);
-	if (s->stream) {
-		g_object_unref (s->stream);
-	}
+	g_clear_object (&s->stream);
+	g_clear_object (&s->input_stream);
 	g_free (s);
 }
 
@@ -1466,8 +1504,8 @@ folder_scan_init (void)
 
 	s = g_malloc (sizeof (*s));
 
-	s->fd = -1;
 	s->stream = NULL;
+	s->input_stream = NULL;
 	s->ioerrno = 0;
 
 	s->outbuf = g_malloc (1024);
@@ -1520,28 +1558,12 @@ folder_scan_reset (struct _header_scan_state *s)
 	s->inend = s->inbuf;
 	s->inptr = s->inbuf;
 	s->inend[0] = '\n';
-	if (s->fd != -1) {
-		close (s->fd);
-		s->fd = -1;
-	}
-	if (s->stream) {
-		g_object_unref (s->stream);
-		s->stream = NULL;
-	}
+	g_clear_object (&s->stream);
+	g_clear_object (&s->input_stream);
 	s->ioerrno = 0;
 	s->eof = FALSE;
 }
 
-static gint
-folder_scan_init_with_fd (struct _header_scan_state *s,
-                          gint fd)
-{
-	folder_scan_reset (s);
-	s->fd = fd;
-
-	return 0;
-}
-
 static gint
 folder_scan_init_with_stream (struct _header_scan_state *s,
                               CamelStream *stream,
