@@ -46,6 +46,7 @@ struct _CamelIMAPXConnManagerPrivate {
 	GList *connections;
 	GWeakRef store;
 	GRWLock rw_lock;
+	guint limit_max_connections;
 };
 
 struct _ConnectionInfo {
@@ -53,6 +54,7 @@ struct _ConnectionInfo {
 	CamelIMAPXServer *is;
 	GHashTable *folder_names;
 	gchar *selected_folder;
+	GError *shutdown_error;
 	volatile gint ref_count;
 };
 
@@ -61,22 +63,15 @@ enum {
 	PROP_STORE
 };
 
-enum {
-	MAILBOX_CREATED,
-	MAILBOX_RENAMED,
-	MAILBOX_UPDATED,
-	LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL];
-
 G_DEFINE_TYPE (
 	CamelIMAPXConnManager,
 	camel_imapx_conn_manager,
 	G_TYPE_OBJECT)
 
 static void
-imapx_conn_shutdown (CamelIMAPXServer *is, CamelIMAPXConnManager *con_man);
+imapx_conn_shutdown (CamelIMAPXServer *is,
+		     const GError *error,
+		     CamelIMAPXConnManager *con_man);
 
 static void
 imapx_conn_update_select (CamelIMAPXServer *is,
@@ -87,31 +82,6 @@ imapx_conn_mailbox_closed (CamelIMAPXServer *is,
 			   CamelIMAPXMailbox *mailbox,
 			   CamelIMAPXConnManager *con_man);
 
-static void
-imapx_conn_mailbox_created_cb (CamelIMAPXServer *server,
-			       CamelIMAPXMailbox *mailbox,
-			       CamelIMAPXConnManager *con_man)
-{
-	g_signal_emit (con_man, signals[MAILBOX_CREATED], 0, mailbox);
-}
-
-static void
-imapx_conn_mailbox_renamed_cb (CamelIMAPXServer *server,
-			       CamelIMAPXMailbox *mailbox,
-			       const gchar *oldname,
-			       CamelIMAPXConnManager *con_man)
-{
-	g_signal_emit (con_man, signals[MAILBOX_RENAMED], 0, mailbox, oldname);
-}
-
-static void
-imapx_conn_mailbox_updated_cb (CamelIMAPXServer *server,
-			       CamelIMAPXMailbox *mailbox,
-			       CamelIMAPXConnManager *con_man)
-{
-	g_signal_emit (con_man, signals[MAILBOX_UPDATED], 0, mailbox);
-}
-
 static ConnectionInfo *
 connection_info_new (CamelIMAPXServer *is)
 {
@@ -128,6 +98,7 @@ connection_info_new (CamelIMAPXServer *is)
 	g_mutex_init (&cinfo->lock);
 	cinfo->is = g_object_ref (is);
 	cinfo->folder_names = folder_names;
+	cinfo->shutdown_error = NULL;
 	cinfo->ref_count = 1;
 
 	return cinfo;
@@ -151,18 +122,16 @@ connection_info_unref (ConnectionInfo *cinfo)
 	g_return_if_fail (cinfo->ref_count > 0);
 
 	if (g_atomic_int_dec_and_test (&cinfo->ref_count)) {
-		camel_imapx_server_shutdown (cinfo->is);
+		camel_imapx_server_shutdown (cinfo->is, cinfo->shutdown_error);
 		g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_shutdown, NULL);
 		g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_update_select, NULL);
 		g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_mailbox_closed, NULL);
-		g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_mailbox_created_cb, NULL);
-		g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_mailbox_renamed_cb, NULL);
-		g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_mailbox_updated_cb, NULL);
 
 		g_mutex_clear (&cinfo->lock);
 		g_object_unref (cinfo->is);
 		g_hash_table_destroy (cinfo->folder_names);
 		g_free (cinfo->selected_folder);
+		g_clear_error (&cinfo->shutdown_error);
 
 		g_slice_free (ConnectionInfo, cinfo);
 	}
@@ -177,10 +146,7 @@ connection_info_cancel_and_unref (ConnectionInfo *cinfo)
 	g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_shutdown, NULL);
 	g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_update_select, NULL);
 	g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_mailbox_closed, NULL);
-	g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_mailbox_created_cb, NULL);
-	g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_mailbox_renamed_cb, NULL);
-	g_signal_handlers_disconnect_matched (cinfo->is, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, imapx_conn_mailbox_updated_cb, NULL);
-	camel_imapx_server_shutdown (cinfo->is);
+	camel_imapx_server_shutdown (cinfo->is, cinfo->shutdown_error);
 	connection_info_unref (cinfo);
 }
 
@@ -283,6 +249,23 @@ connection_info_set_selected_folder (ConnectionInfo *cinfo,
 	g_mutex_unlock (&cinfo->lock);
 }
 
+static void
+connection_info_set_shutdown_error (ConnectionInfo *cinfo,
+                                    const GError *shutdown_error)
+{
+	g_return_if_fail (cinfo != NULL);
+
+	g_mutex_lock (&cinfo->lock);
+
+	if (cinfo->shutdown_error != shutdown_error) {
+		g_clear_error (&cinfo->shutdown_error);
+		if (shutdown_error)
+			cinfo->shutdown_error = g_error_copy (shutdown_error);
+	}
+
+	g_mutex_unlock (&cinfo->lock);
+}
+
 static GList *
 imapx_conn_manager_list_info (CamelIMAPXConnManager *con_man)
 {
@@ -455,34 +438,6 @@ camel_imapx_conn_manager_class_init (CamelIMAPXConnManagerClass *class)
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT_ONLY |
 			G_PARAM_STATIC_STRINGS));
-
-	signals[MAILBOX_CREATED] = g_signal_new (
-		"mailbox-created",
-		G_OBJECT_CLASS_TYPE (class),
-		G_SIGNAL_RUN_FIRST,
-		G_STRUCT_OFFSET (CamelIMAPXConnManagerClass, mailbox_created),
-		NULL, NULL, NULL,
-		G_TYPE_NONE, 1,
-		CAMEL_TYPE_IMAPX_MAILBOX);
-
-	signals[MAILBOX_RENAMED] = g_signal_new (
-		"mailbox-renamed",
-		G_OBJECT_CLASS_TYPE (class),
-		G_SIGNAL_RUN_FIRST,
-		G_STRUCT_OFFSET (CamelIMAPXConnManagerClass, mailbox_renamed),
-		NULL, NULL, NULL,
-		G_TYPE_NONE, 2,
-		CAMEL_TYPE_IMAPX_MAILBOX,
-		G_TYPE_STRING);
-
-	signals[MAILBOX_UPDATED] = g_signal_new (
-		"mailbox-updated",
-		G_OBJECT_CLASS_TYPE (class),
-		G_SIGNAL_RUN_FIRST,
-		G_STRUCT_OFFSET (CamelIMAPXConnManagerClass, mailbox_updated),
-		NULL, NULL, NULL,
-		G_TYPE_NONE, 1,
-		CAMEL_TYPE_IMAPX_MAILBOX);
 }
 
 static void
@@ -493,11 +448,9 @@ camel_imapx_conn_manager_init (CamelIMAPXConnManager *con_man)
 	g_rw_lock_init (&con_man->priv->rw_lock);
 }
 
-/* Static functions go here */
-
-/* TODO destroy unused connections in a time-out loop */
 static void
 imapx_conn_shutdown (CamelIMAPXServer *is,
+		     const GError *error,
                      CamelIMAPXConnManager *con_man)
 {
 	ConnectionInfo *cinfo;
@@ -509,6 +462,14 @@ imapx_conn_shutdown (CamelIMAPXServer *is,
 		imapx_conn_manager_remove_info (con_man, cinfo);
 		connection_info_unref (cinfo);
 	}
+
+	/* If one connection ends with this error, then it means all
+	   other opened connections also may end with the same error,
+	   thus better to kill them all from the list of connections.
+	*/
+	if (g_error_matches (error, CAMEL_IMAPX_SERVER_ERROR, CAMEL_IMAPX_SERVER_ERROR_TRY_RECONNECT)) {
+		camel_imapx_conn_manager_close_connections (con_man, error);
+	}
 }
 
 static void
@@ -577,6 +538,10 @@ imapx_find_connection_unlocked (CamelIMAPXConnManager *con_man,
 		camel_imapx_settings_get_concurrent_connections (
 		CAMEL_IMAPX_SETTINGS (settings));
 
+	if (con_man->priv->limit_max_connections > 0 &&
+	    con_man->priv->limit_max_connections < concurrent_connections)
+		concurrent_connections = con_man->priv->limit_max_connections;
+
 	g_object_unref (settings);
 
 	/* XXX Have a dedicated connection for INBOX ? */
@@ -596,7 +561,7 @@ imapx_find_connection_unlocked (CamelIMAPXConnManager *con_man,
 		if (camel_imapx_server_has_expensive_command (candidate->is))
 			expensive_connections++;
 
-		if (connection_info_has_folder_name (candidate, folder_name) &&
+		if (connection_info_has_folder_name (candidate, folder_name) && camel_imapx_server_is_connected (candidate->is) &&
 		    (opened_connections >= concurrent_connections || for_expensive_job || !camel_imapx_server_has_expensive_command (candidate->is))) {
 			if (cinfo) {
 				/* group expensive jobs into one connection */
@@ -634,7 +599,8 @@ imapx_find_connection_unlocked (CamelIMAPXConnManager *con_man,
 			ConnectionInfo *candidate = link->data;
 			guint jobs;
 
-			if (!camel_imapx_server_has_expensive_command (candidate->is))
+			if (!camel_imapx_server_is_connected (candidate->is) ||
+			    !camel_imapx_server_has_expensive_command (candidate->is))
 				continue;
 
 			jobs = camel_imapx_server_get_command_count (candidate->is);
@@ -658,7 +624,8 @@ imapx_find_connection_unlocked (CamelIMAPXConnManager *con_man,
 	for (link = list; link != NULL; link = g_list_next (link)) {
 		ConnectionInfo *candidate = link->data;
 
-		if (connection_info_is_available (candidate)) {
+		if (camel_imapx_server_is_connected (candidate->is) &&
+		    connection_info_is_available (candidate)) {
 			if (cinfo)
 				connection_info_unref (cinfo);
 			cinfo = connection_info_ref (candidate);
@@ -681,7 +648,12 @@ imapx_find_connection_unlocked (CamelIMAPXConnManager *con_man,
 	/* Pick the connection with the least number of jobs in progress. */
 	for (link = list; link != NULL; link = g_list_next (link)) {
 		ConnectionInfo *candidate = link->data;
-		gint n_commands = camel_imapx_server_get_command_count (candidate->is);
+		gint n_commands;
+
+		if (!camel_imapx_server_is_connected (candidate->is))
+			continue;
+
+		n_commands = camel_imapx_server_get_command_count (candidate->is);
 
 		if (cinfo == NULL) {
 			cinfo = connection_info_ref (candidate);
@@ -703,7 +675,7 @@ exit:
 		for (link = list; link != NULL; link = g_list_next (link)) {
 			ConnectionInfo *candidate = link->data;
 
-			printf ("   cmds:%d has-expensive:%d avail:%d cinfo:%p\n", camel_imapx_server_get_command_count (candidate->is), camel_imapx_server_has_expensive_command (candidate->is), connection_info_is_available (candidate), candidate);
+			printf ("   cmds:%d has-expensive:%d avail:%d cinfo:%p server:%p\n", camel_imapx_server_get_command_count (candidate->is), camel_imapx_server_has_expensive_command (candidate->is), connection_info_is_available (candidate), candidate, candidate->is);
 		}
 	}
 
@@ -757,9 +729,9 @@ imapx_create_new_connection_unlocked (CamelIMAPXConnManager *con_man,
 	 *     we should not have multiple IMAPX connections trying to
 	 *     authenticate at once, so this should be thread-safe.
 	 */
-	camel_imapx_store_set_connecting_server (imapx_store, is);
+	camel_imapx_store_set_connecting_server (imapx_store, is, con_man->priv->connections != NULL);
 	success = camel_imapx_server_connect (is, cancellable, error);
-	camel_imapx_store_set_connecting_server (imapx_store, NULL);
+	camel_imapx_store_set_connecting_server (imapx_store, NULL, FALSE);
 
 	if (!success) {
 		g_clear_object (&is);
@@ -777,18 +749,6 @@ imapx_create_new_connection_unlocked (CamelIMAPXConnManager *con_man,
 		is, "mailbox-closed",
 		G_CALLBACK (imapx_conn_mailbox_closed), con_man);
 
-	g_signal_connect (
-		is, "mailbox-created",
-		G_CALLBACK (imapx_conn_mailbox_created_cb), con_man);
-
-	g_signal_connect (
-		is, "mailbox-renamed",
-		G_CALLBACK (imapx_conn_mailbox_renamed_cb), con_man);
-
-	g_signal_connect (
-		is, "mailbox-updated",
-		G_CALLBACK (imapx_conn_mailbox_updated_cb), con_man);
-
 	cinfo = connection_info_new (is);
 
 	if (folder_name != NULL)
@@ -798,7 +758,7 @@ imapx_create_new_connection_unlocked (CamelIMAPXConnManager *con_man,
 	con_man->priv->connections = g_list_prepend (
 		con_man->priv->connections, cinfo);
 
-	c (is->tagprefix, "Created new connection for %s and total connections %d \n", folder_name, g_list_length (con_man->priv->connections));
+	c (is->tagprefix, "Created new connection %p (server:%p) for %s; total connections %d\n", cinfo, cinfo->is, folder_name, g_list_length (con_man->priv->connections));
 
 exit:
 	g_object_unref (store);
@@ -843,9 +803,35 @@ camel_imapx_conn_manager_get_connection (CamelIMAPXConnManager *con_man,
 	/* Check if we got cancelled while waiting for the lock. */
 	if (!g_cancellable_set_error_if_cancelled (cancellable, error)) {
 		is = imapx_find_connection_unlocked (con_man, folder_name, for_expensive_job);
-		if (is == NULL)
-			is = imapx_create_new_connection_unlocked (
-				con_man, folder_name, cancellable, error);
+		if (is == NULL) {
+			GError *local_error = NULL;
+
+			is = imapx_create_new_connection_unlocked (con_man, folder_name, cancellable, &local_error);
+
+			if (!is) {
+				gboolean limit_connections =
+					g_error_matches (local_error, CAMEL_IMAPX_SERVER_ERROR,
+					CAMEL_IMAPX_SERVER_ERROR_CONCURRENT_CONNECT_FAILED) &&
+					con_man->priv->connections;
+
+				c ('*', "Failed to open a new connection, while having %d opened, with error: %s; will limit connections: %s\n",
+					g_list_length (con_man->priv->connections),
+					local_error ? local_error->message : "Unknown error",
+					limit_connections ? "yes" : "no");
+
+				if (limit_connections) {
+					/* limit to one-less than current connection count - be nice to the server */
+					con_man->priv->limit_max_connections = g_list_length (con_man->priv->connections) - 1;
+					if (!con_man->priv->limit_max_connections)
+						con_man->priv->limit_max_connections = 1;
+
+					g_clear_error (&local_error);
+					is = imapx_find_connection_unlocked (con_man, folder_name, for_expensive_job);
+				} else if (local_error) {
+					g_propagate_error (error, local_error);
+				}
+			}
+		}
 	}
 
 	CON_WRITE_UNLOCK (con_man);
@@ -896,45 +882,26 @@ camel_imapx_conn_manager_update_con_info (CamelIMAPXConnManager *con_man,
 	connection_info_unref (cinfo);
 }
 
-CamelIMAPXMailbox *
-camel_imapx_conn_manager_ref_mailbox (CamelIMAPXConnManager *con_man,
-				      const gchar *mailbox_name)
-{
-	CamelIMAPXMailbox *mailbox = NULL;
-	GList *iter;
-
-	g_return_val_if_fail (CAMEL_IS_IMAPX_CONN_MANAGER (con_man), NULL);
-	g_return_val_if_fail (mailbox_name != NULL, NULL);
-
-	CON_READ_LOCK (con_man);
-
-	for (iter = con_man->priv->connections; iter != NULL; iter = g_list_next (iter)) {
-		ConnectionInfo *candidate = iter->data;
-
-		if (candidate->is) {
-			mailbox = camel_imapx_server_ref_mailbox (candidate->is, mailbox_name);
-			if (mailbox)
-				break;
-		}
-	}
-
-	CON_READ_UNLOCK (con_man);
-
-	return mailbox;
-}
-
 void
-camel_imapx_conn_manager_close_connections (CamelIMAPXConnManager *con_man)
+camel_imapx_conn_manager_close_connections (CamelIMAPXConnManager *con_man,
+					    const GError *error)
 {
+	GList *iter, *connections;
+
 	g_return_if_fail (CAMEL_IS_IMAPX_CONN_MANAGER (con_man));
 
 	CON_WRITE_LOCK (con_man);
 
-	g_list_free_full (
-		con_man->priv->connections,
-		(GDestroyNotify) connection_info_cancel_and_unref);
+	c('*', "Closing all %d connections, with propagated error: %s\n", g_list_length (con_man->priv->connections), error ? error->message : "none");
+
+	connections = con_man->priv->connections;
 	con_man->priv->connections = NULL;
 
 	CON_WRITE_UNLOCK (con_man);
-}
 
+	for (iter = connections; iter; iter = g_list_next (iter)) {
+		connection_info_set_shutdown_error (iter->data, error);
+	}
+
+	g_list_free_full (connections, (GDestroyNotify) connection_info_cancel_and_unref);
+}
