@@ -56,15 +56,6 @@ struct _CamelIMAPXConnManager {
 
 struct _CamelIMAPXConnManagerClass {
 	GObjectClass parent_class;
-
-	/* Signals */
-	void		(*mailbox_created)	(CamelIMAPXConnManager *con_man,
-						 CamelIMAPXMailbox *mailbox);
-	void		(*mailbox_renamed)	(CamelIMAPXConnManager *con_man,
-						 CamelIMAPXMailbox *mailbox,
-						 const gchar *oldname);
-	void		(*mailbox_updated)	(CamelIMAPXConnManager *con_man,
-						 CamelIMAPXMailbox *mailbox);
 };
 
 GType		camel_imapx_conn_manager_get_type (void);
@@ -80,17 +71,14 @@ CamelIMAPXServer *
 						 GCancellable *cancellable,
 						 GError **error);
 void		camel_imapx_conn_manager_close_connections
-						(CamelIMAPXConnManager *con_man);
+						(CamelIMAPXConnManager *con_man,
+						 const GError *error);
 GList *		camel_imapx_conn_manager_get_connections
 						(CamelIMAPXConnManager *con_man);
 void		camel_imapx_conn_manager_update_con_info
 						(CamelIMAPXConnManager *con_man,
 						 CamelIMAPXServer *server,
 						 const gchar *folder_name);
-CamelIMAPXMailbox *
-		camel_imapx_conn_manager_ref_mailbox
-						(CamelIMAPXConnManager *con_man,
-						 const gchar *mailbox_name);
 
 G_END_DECLS
 
