@@ -44,11 +44,21 @@
 	(G_TYPE_INSTANCE_GET_CLASS \
 	((obj), CAMEL_TYPE_IMAPX_SERVER, CamelIMAPXServerClass))
 
+#define CAMEL_IMAPX_SERVER_ERROR (camel_imapx_server_error_quark ())
+
 G_BEGIN_DECLS
 
+typedef enum {
+	CAMEL_IMAPX_SERVER_ERROR_CONCURRENT_CONNECT_FAILED,
+	CAMEL_IMAPX_SERVER_ERROR_TRY_RECONNECT
+} CamelIMAPXServerError;
+
+GQuark		camel_imapx_server_error_quark		(void) G_GNUC_CONST;
+
 /* Avoid a circular reference. */
 struct _CamelIMAPXStore;
 struct _CamelIMAPXSettings;
+struct _CamelIMAPXJob;
 
 typedef struct _CamelIMAPXServer CamelIMAPXServer;
 typedef struct _CamelIMAPXServerClass CamelIMAPXServerClass;
@@ -126,14 +136,8 @@ struct _CamelIMAPXServerClass {
 						 CamelIMAPXMailbox *mailbox);
 	void		(*mailbox_closed)	(CamelIMAPXServer *is,
 						 CamelIMAPXMailbox *mailbox);
-	void		(*mailbox_created)	(CamelIMAPXServer *is,
-						 CamelIMAPXMailbox *mailbox);
-	void		(*mailbox_renamed)	(CamelIMAPXServer *is,
-						 CamelIMAPXMailbox *mailbox,
-						 const gchar *oldname);
-	void		(*mailbox_updated)	(CamelIMAPXServer *is,
-						 CamelIMAPXMailbox *mailbox);
-	void		(*shutdown)		(CamelIMAPXServer *is);
+	void		(*shutdown)		(CamelIMAPXServer *is,
+						 const GError *error);
 };
 
 GType		camel_imapx_server_get_type	(void);
@@ -147,30 +151,22 @@ GInputStream *	camel_imapx_server_ref_input_stream
 						(CamelIMAPXServer *is);
 GOutputStream *	camel_imapx_server_ref_output_stream
 						(CamelIMAPXServer *is);
-CamelIMAPXNamespaceResponse *
-		camel_imapx_server_ref_namespaces
-						(CamelIMAPXServer *is);
-CamelIMAPXMailbox *
-		camel_imapx_server_ref_mailbox	(CamelIMAPXServer *is,
-						 const gchar *mailbox_name);
 CamelIMAPXMailbox *
 		camel_imapx_server_ref_selected	(CamelIMAPXServer *is);
-GList *		camel_imapx_server_list_mailboxes
-						(CamelIMAPXServer *is,
-						 CamelIMAPXNamespace *namespace_,
-						 const gchar *pattern);
 gboolean	camel_imapx_server_connect	(CamelIMAPXServer *is,
 						 GCancellable *cancellable,
 						 GError **error);
 gboolean	imapx_connect_to_server		(CamelIMAPXServer *is,
 						 GCancellable *cancellable,
 						 GError **error);
+gboolean	camel_imapx_server_is_connected	(CamelIMAPXServer *imapx_server);
 CamelAuthenticationResult
 		camel_imapx_server_authenticate	(CamelIMAPXServer *is,
 						 const gchar *mechanism,
 						 GCancellable *cancellable,
 						 GError **error);
-void		camel_imapx_server_shutdown	(CamelIMAPXServer *is);
+void		camel_imapx_server_shutdown	(CamelIMAPXServer *is,
+						 const GError *error);
 gboolean	camel_imapx_server_list		(CamelIMAPXServer *is,
 						 const gchar *pattern,
 						 CamelStoreGetFolderInfoFlags flags,
@@ -272,6 +268,11 @@ const CamelIMAPXUntaggedRespHandlerDesc *
 						(CamelIMAPXServer *is,
 						 const gchar *untagged_response,
 						 const CamelIMAPXUntaggedRespHandlerDesc *desc);
+struct _CamelIMAPXJob *
+		camel_imapx_server_ref_job	(CamelIMAPXServer *imapx_server,
+						 CamelIMAPXMailbox *mailbox,
+						 guint32 job_type,
+						 const gchar *uid);
 
 G_END_DECLS
 
