@@ -47,6 +47,9 @@
 
 G_BEGIN_DECLS
 
+/* Avoid a circular reference. */
+struct _CamelIMAPXJob;
+
 typedef struct _CamelIMAPXStore CamelIMAPXStore;
 typedef struct _CamelIMAPXStoreClass CamelIMAPXStoreClass;
 typedef struct _CamelIMAPXStorePrivate CamelIMAPXStorePrivate;
@@ -60,6 +63,15 @@ struct _CamelIMAPXStore {
 
 struct _CamelIMAPXStoreClass {
 	CamelOfflineStoreClass parent_class;
+
+	/* Signals */
+	void		(*mailbox_created)	(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXMailbox *mailbox);
+	void		(*mailbox_renamed)	(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXMailbox *mailbox,
+						 const gchar *oldname);
+	void		(*mailbox_updated)	(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXMailbox *mailbox);
 };
 
 GType		camel_imapx_store_get_type	(void);
@@ -71,15 +83,42 @@ CamelIMAPXServer *
 						 GError **error);
 void		camel_imapx_store_set_connecting_server
 						(CamelIMAPXStore *store,
-						 CamelIMAPXServer *server);
+						 CamelIMAPXServer *server,
+						 gboolean is_concurrent_connection);
+gboolean	camel_imapx_store_is_connecting_concurrent_connection
+						(CamelIMAPXStore *imapx_store);
 void		camel_imapx_store_folder_op_done
 						(CamelIMAPXStore *store,
 						 CamelIMAPXServer *server,
 						 const gchar *folder_name);
+CamelIMAPXNamespaceResponse *
+		camel_imapx_store_ref_namespaces
+						(CamelIMAPXStore *imapx_store);
+void		camel_imapx_store_set_namespaces
+						(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXNamespaceResponse *namespaces);
 CamelIMAPXMailbox *
 		camel_imapx_store_ref_mailbox	(CamelIMAPXStore *imapx_store,
 						 const gchar *mailbox_name);
-
+GList *		camel_imapx_store_list_mailboxes
+						(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXNamespace *namespace_,
+						 const gchar *pattern);
+void		camel_imapx_store_emit_mailbox_updated
+						(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXMailbox *mailbox);
+void		camel_imapx_store_handle_mailbox_rename
+						(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXMailbox *old_mailbox,
+						 const gchar *new_mailbox_name);
+void		camel_imapx_store_handle_list_response
+						(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXServer *imapx_server,
+						 CamelIMAPXListResponse *response);
+void		camel_imapx_store_handle_lsub_response
+						(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXServer *imapx_server,
+						 CamelIMAPXListResponse *response);
 CamelFolderQuotaInfo *
 		camel_imapx_store_dup_quota_info
 						(CamelIMAPXStore *store,
@@ -88,6 +127,11 @@ void		camel_imapx_store_set_quota_info
 						(CamelIMAPXStore *store,
 						 const gchar *quota_root_name,
 						 const CamelFolderQuotaInfo *info);
+struct _CamelIMAPXJob *
+		camel_imapx_store_ref_job	(CamelIMAPXStore *imapx_store,
+						 CamelIMAPXMailbox *mailbox,
+						 guint32 job_type,
+						 const gchar *uid);
 
 G_END_DECLS
 
